@@ -1,6 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2020 The Music Player Daemon Project
- * Project homepage: http://musicpd.org
+ * Copyright 2004-2021 The Music Player Daemon Project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +20,7 @@
 #include "FileListPage.hxx"
 #include "FileBrowserPage.hxx"
 #include "SongPage.hxx"
+#include "EditPlaylistPage.hxx"
 #include "LyricsPage.hxx"
 #include "Command.hxx"
 #include "screen_status.hxx"
@@ -45,7 +45,7 @@
 #define BUFSIZE 1024
 
 #ifndef NCMPC_MINI
-#define HIGHLIGHT  (0x01)
+static constexpr unsigned HIGHLIGHT = 0x01;
 #endif
 
 FileListPage::FileListPage(ScreenManager &_screen, WINDOW *_w,
@@ -291,7 +291,7 @@ browser_select_entry(struct mpdclient &c, FileListEntry &entry,
 	return true;
 }
 
-bool
+inline bool
 FileListPage::HandleSelect(struct mpdclient &c)
 {
 	bool success = false;
@@ -308,7 +308,7 @@ FileListPage::HandleSelect(struct mpdclient &c)
 	return range.end_index == range.start_index + 1 && success;
 }
 
-bool
+inline bool
 FileListPage::HandleAdd(struct mpdclient &c)
 {
 	bool success = false;
@@ -324,7 +324,26 @@ FileListPage::HandleAdd(struct mpdclient &c)
 	return range.end_index == range.start_index + 1 && success;
 }
 
-void
+#ifdef ENABLE_PLAYLIST_EDITOR
+
+inline bool
+FileListPage::HandleEdit(struct mpdclient &c)
+{
+	const auto *entity = GetSelectedEntity();
+	if (entity == nullptr ||
+	    mpd_entity_get_type(entity) != MPD_ENTITY_TYPE_PLAYLIST)
+		return false;
+
+	const auto *playlist = mpd_entity_get_playlist(entity);
+	assert(playlist != nullptr);
+
+	EditPlaylist(screen, c, mpd_playlist_get_path(playlist));
+	return true;
+}
+
+#endif
+
+inline void
 FileListPage::HandleSelectAll(struct mpdclient &c)
 {
 	if (filelist == nullptr)
@@ -353,11 +372,13 @@ FileListPage::OnMouse(struct mpdclient &c, Point p,
 
 	lw.SetCursorFromOrigin(p.y);
 
-	if( bstate & BUTTON1_CLICKED ) {
-		if (prev_selected == lw.GetCursorIndex())
+	if (bstate & (BUTTON1_CLICKED|BUTTON1_DOUBLE_CLICKED)) {
+		if ((bstate & BUTTON1_DOUBLE_CLICKED) ||
+		    prev_selected == lw.GetCursorIndex())
 			HandleEnter(c);
-	} else if (bstate & BUTTON3_CLICKED) {
-		if (prev_selected == lw.GetCursorIndex())
+	} else if (bstate & (BUTTON3_CLICKED|BUTTON3_DOUBLE_CLICKED)) {
+		if ((bstate & BUTTON3_DOUBLE_CLICKED) ||
+		    prev_selected == lw.GetCursorIndex())
 			HandleSelect(c);
 	}
 
@@ -443,6 +464,13 @@ FileListPage::OnCommand(struct mpdclient &c, Command cmd)
 		SetDirty();
 		return true;
 
+#ifdef ENABLE_PLAYLIST_EDITOR
+	case Command::EDIT:
+		if (HandleEdit(c))
+			return true;
+		break;
+#endif
+
 	case Command::SELECT_ALL:
 		HandleSelectAll(c);
 		return true;
@@ -501,7 +529,7 @@ FileListPage::PaintListItem(WINDOW *w, unsigned i,
 #ifndef NCMPC_MINI
 	const bool highlight = (entry.flags & HIGHLIGHT) != 0;
 #else
-	const bool highlight = false;
+	constexpr bool highlight = false;
 #endif
 
 	switch (mpd_entity_get_type(entity)) {
