@@ -17,6 +17,8 @@
  */
 
 #include "LyricsPage.hxx"
+#include "LyricsCache.hxx"
+#include "LyricsLoader.hxx"
 #include "PageMeta.hxx"
 #include "screen_status.hxx"
 #include "FileBrowserPage.hxx"
@@ -26,7 +28,6 @@
 #include "Options.hxx"
 #include "mpdclient.hxx"
 #include "screen.hxx"
-#include "lyrics.hxx"
 #include "plugin.hxx"
 #include "TextPage.hxx"
 #include "screen_utils.hxx"
@@ -36,11 +37,9 @@
 
 #include <assert.h>
 #include <errno.h>
-#include <sys/stat.h>
 #include <sys/wait.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <stdio.h>
 
 static struct mpd_song *next_song;
@@ -58,16 +57,20 @@ class LyricsPage final : public TextPage, PluginResponseHandler {
 	 */
 	const char *artist = nullptr, *title = nullptr;
 
+	LyricsCache cache;
+
 	std::string plugin_name;
 
-	PluginCycle *loader = nullptr;
+	LyricsLoader loader;
 
-	TimerEvent loader_timeout;
+	PluginCycle *plugin_cycle = nullptr;
+
+	CoarseTimerEvent plugin_timeout;
 
 public:
 	LyricsPage(ScreenManager &_screen, WINDOW *w, Size size)
 		:TextPage(_screen, w, size),
-		 loader_timeout(_screen.GetEventLoop(),
+		 plugin_timeout(_screen.GetEventLoop(),
 				BIND_THIS_METHOD(OnTimeout)) {}
 
 	~LyricsPage() override {
@@ -75,10 +78,17 @@ public:
 	}
 
 	auto &GetEventLoop() noexcept {
-		return loader_timeout.GetEventLoop();
+		return plugin_timeout.GetEventLoop();
 	}
 
 private:
+	void StopPluginCycle() noexcept {
+		assert(plugin_cycle != nullptr);
+
+		plugin_stop(*plugin_cycle);
+		plugin_cycle = nullptr;
+	}
+
 	void Cancel();
 
 	/**
@@ -93,6 +103,8 @@ private:
 
 	void Set(const char *s);
 
+	void StartPluginCycle() noexcept;
+
 	void Load(const struct mpd_song &song) noexcept;
 	void MaybeLoad(const struct mpd_song &new_song) noexcept;
 
@@ -104,7 +116,6 @@ private:
 	void Reload();
 
 	bool Save();
-	bool Delete();
 
 	/** save current lyrics to a file and run editor on it */
 	void Edit();
@@ -128,12 +139,10 @@ private:
 void
 LyricsPage::Cancel()
 {
-	if (loader != nullptr) {
-		plugin_stop(loader);
-		loader = nullptr;
-	}
+	if (plugin_cycle != nullptr)
+		StopPluginCycle();
 
-	loader_timeout.Cancel();
+	plugin_timeout.Cancel();
 
 	plugin_name.clear();
 
@@ -145,41 +154,10 @@ LyricsPage::Cancel()
 	}
 }
 
-static void
-path_lyr_file(char *path, size_t size,
-		const char *artist, const char *title)
-{
-	snprintf(path, size, "%s/.lyrics/%s - %s.txt",
-			getenv("HOME"), artist, title);
-}
-
-static bool
-exists_lyr_file(const char *artist, const char *title)
-{
-	char path[1024];
-	path_lyr_file(path, 1024, artist, title);
-
-	struct stat result;
-	return (stat(path, &result) == 0);
-}
-
-static FILE *
-create_lyr_file(const char *artist, const char *title)
-{
-	char path[1024];
-	snprintf(path, 1024, "%s/.lyrics",
-		 getenv("HOME"));
-	mkdir(path, S_IRWXU);
-
-	path_lyr_file(path, 1024, artist, title);
-
-	return fopen(path, "w");
-}
-
 bool
 LyricsPage::Save()
 {
-	FILE *lyr_file = create_lyr_file(artist, title);
+	FILE *lyr_file = cache.Save(artist, title);
 	if (lyr_file == nullptr)
 		return false;
 
@@ -190,17 +168,6 @@ LyricsPage::Save()
 	return true;
 }
 
-bool
-LyricsPage::Delete()
-{
-	if (!exists_lyr_file(artist, title))
-		return false;
-
-	char path[1024];
-	path_lyr_file(path, 1024, artist, title);
-	return unlink(path) == 0;
-}
-
 void
 LyricsPage::Set(const char *s)
 {
@@ -231,13 +198,16 @@ LyricsPage::OnPluginSuccess(const char *_plugin_name,
 
 	Set(result.c_str());
 
-	if (options.lyrics_autosave && !exists_lyr_file(artist, title))
+	if (options.lyrics_autosave && cache.IsAvailable() &&
+	    !cache.Exists(artist, title))
 		Save();
 
-	loader_timeout.Cancel();
+	plugin_timeout.Cancel();
 
-	plugin_stop(loader);
-	loader = nullptr;
+	StopPluginCycle();
+
+	/* schedule a full repaint so the page title gets updated */
+	screen.SchedulePaint();
 }
 
 void
@@ -250,20 +220,36 @@ LyricsPage::OnPluginError(std::string error) noexcept
 	/* translators: no lyrics were found for the song */
 	screen_status_message(_("No lyrics"));
 
-	loader_timeout.Cancel();
+	plugin_timeout.Cancel();
+	StopPluginCycle();
 
-	plugin_stop(loader);
-	loader = nullptr;
+	/* schedule a full repaint so the page title gets updated */
+	screen.SchedulePaint();
 }
 
 void
 LyricsPage::OnTimeout() noexcept
 {
-	plugin_stop(loader);
-	loader = nullptr;
+	StopPluginCycle();
 
 	screen_status_printf(_("Lyrics timeout occurred after %d seconds"),
 			     (int)std::chrono::duration_cast<std::chrono::seconds>(options.lyrics_timeout).count());
+
+	/* schedule a full repaint so the page title gets updated */
+	screen.SchedulePaint();
+}
+
+void
+LyricsPage::StartPluginCycle() noexcept
+{
+	assert(artist != nullptr);
+	assert(title != nullptr);
+	assert(plugin_cycle == nullptr);
+
+	plugin_cycle = loader.Load(GetEventLoop(), artist, title, *this);
+
+	if (options.lyrics_timeout > std::chrono::steady_clock::duration::zero())
+		plugin_timeout.Schedule(options.lyrics_timeout);
 }
 
 void
@@ -281,11 +267,13 @@ LyricsPage::Load(const struct mpd_song &_song) noexcept
 		return;
 	}
 
-	loader = lyrics_load(GetEventLoop(),
-			     artist, title, *this);
-
-	if (options.lyrics_timeout > std::chrono::steady_clock::duration::zero())
-		loader_timeout.Schedule(options.lyrics_timeout);
+	if (auto from_cache = cache.Load(artist, title); !from_cache.empty()) {
+		/* cached */
+		plugin_name = "cache";
+		Set(from_cache.c_str());
+	} else
+		/* not cached - invoke plugins */
+		StartPluginCycle();
 }
 
 void
@@ -300,10 +288,9 @@ LyricsPage::MaybeLoad(const struct mpd_song &new_song) noexcept
 void
 LyricsPage::Reload()
 {
-	if (loader == nullptr && artist != nullptr && title != nullptr) {
+	if (plugin_cycle == nullptr && artist != nullptr && title != nullptr) {
 		reloading = true;
-		loader = lyrics_load(GetEventLoop(),
-				     artist, title, *this);
+		StartPluginCycle();
 		Repaint();
 	}
 }
@@ -338,7 +325,7 @@ LyricsPage::Update(struct mpdclient &c, unsigned) noexcept
 const char *
 LyricsPage::GetTitle(char *str, size_t size) const noexcept
 {
-	if (loader != nullptr) {
+	if (plugin_cycle != nullptr) {
 		snprintf(str, size, "%s (%s)",
 			 _("Lyrics"),
 			 /* translators: this message is displayed
@@ -364,6 +351,12 @@ LyricsPage::GetTitle(char *str, size_t size) const noexcept
 void
 LyricsPage::Edit()
 {
+	const auto path = cache.MakePath(artist, title);
+	if (path.empty()) {
+		screen_status_message(_("Lyrics cache is unavailable"));
+		return;
+	}
+
 	if (options.text_editor.empty()) {
 		screen_status_message(_("Editor not configured"));
 		return;
@@ -394,9 +387,7 @@ LyricsPage::Edit()
 		ncu_init();
 		return;
 	} else if (pid == 0) {
-		char path[1024];
-		path_lyr_file(path, sizeof(path), artist, title);
-		execlp(editor, editor, path, nullptr);
+		execlp(editor, editor, path.c_str(), nullptr);
 		/* exec failed, do what system does */
 		_exit(127);
 	} else {
@@ -434,21 +425,21 @@ LyricsPage::OnCommand(struct mpdclient &c, Command cmd)
 
 	switch(cmd) {
 	case Command::INTERRUPT:
-		if (loader != nullptr) {
+		if (plugin_cycle != nullptr) {
 			Cancel();
 			Clear();
 		}
 		return true;
 	case Command::SAVE_PLAYLIST:
-		if (loader == nullptr && artist != nullptr &&
+		if (plugin_cycle == nullptr && artist != nullptr &&
 		    title != nullptr && Save())
 			/* lyrics for the song were saved on hard disk */
 			screen_status_message (_("Lyrics saved"));
 		return true;
 	case Command::DELETE:
-		if (loader == nullptr && artist != nullptr &&
+		if (plugin_cycle == nullptr && artist != nullptr &&
 		    title != nullptr) {
-			screen_status_message(Delete()
+			screen_status_message(cache.Delete(artist, title)
 					      ? _("Lyrics deleted")
 					      : _("No saved lyrics"));
 		}
