@@ -42,10 +42,6 @@
 #include "ConfigFile.hxx"
 #endif
 
-#ifdef ENABLE_LYRICS_SCREEN
-#include "lyrics.hxx"
-#endif
-
 #include <mpd/client.h>
 
 #include <curses.h>
@@ -319,10 +315,6 @@ try {
 
 	const ScopeCursesInit curses_init;
 
-#ifdef ENABLE_LYRICS_SCREEN
-	lyrics_init();
-#endif
-
 	/* create the global Instance */
 	Instance instance;
 	global_instance = &instance;
