@@ -41,7 +41,7 @@
 #include "SongPage.hxx"
 #include "LyricsPage.hxx"
 #include "db_completion.hxx"
-#include "event/TimerEvent.hxx"
+#include "event/CoarseTimerEvent.hxx"
 #include "util/Compiler.h"
 
 #ifndef NCMPC_MINI
@@ -64,7 +64,7 @@ class QueuePage final : public ListPage, ListRenderer, ListText {
 	mutable class hscroll hscroll;
 #endif
 
-	TimerEvent hide_cursor_timer;
+	CoarseTimerEvent hide_cursor_timer;
 
 	MpdQueue *playlist = nullptr;
 	int current_song_id = -1;
@@ -333,7 +333,7 @@ QueuePage::OnHideCursorTimer() noexcept
 	/* hide the cursor when mpd is playing and the user is inactive */
 
 	if (playing) {
-		lw.DisableCursor();
+		lw.HideCursor();
 		Repaint();
 	} else
 		ScheduleHideCursor();
@@ -345,7 +345,7 @@ QueuePage::OnOpen(struct mpdclient &c) noexcept
 	playlist = &c.playlist;
 
 	if (options.hide_cursor > std::chrono::steady_clock::duration::zero()) {
-		lw.EnableCursor();
+		lw.ShowCursor();
 		ScheduleHideCursor();
 	}
 
@@ -517,7 +517,7 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 	const Command prev_cmd = cached_cmd;
 	cached_cmd = cmd;
 
-	lw.EnableCursor();
+	lw.ShowCursor();
 
 	if (options.hide_cursor > std::chrono::steady_clock::duration::zero()) {
 		ScheduleHideCursor();
