@@ -123,7 +123,7 @@ public:
 	SongPage(ScreenManager &_screen, WINDOW *w, Size size) noexcept
 		:ListPage(w, size),
 		 screen(_screen) {
-		lw.DisableCursor();
+		lw.HideCursor();
 	}
 
 	~SongPage() noexcept override {
@@ -549,8 +549,6 @@ SongPage::OnCommand(struct mpdclient &c, Command cmd)
 	}
 
 	if (screen_find(screen, lw, cmd, *this)) {
-		/* center the row */
-		lw.Center(lw.GetCursorIndex());
 		SetDirty();
 		return true;
 	}
