@@ -18,7 +18,6 @@
  */
 
 #include "Loop.hxx"
-#include "TimerEvent.hxx"
 #include "DeferEvent.hxx"
 #include "SocketEvent.hxx"
 #include "IdleEvent.hxx"
@@ -34,13 +33,6 @@
 #include <stdio.h>
 #endif
 
-constexpr bool
-EventLoop::TimerCompare::operator()(const TimerEvent &a,
-				    const TimerEvent &b) const noexcept
-{
-	return a.due < b.due;
-}
-
 EventLoop::EventLoop(
 #ifdef HAVE_THREADED_EVENT_LOOP
 		     ThreadId _thread
@@ -60,7 +52,6 @@ EventLoop::EventLoop(
 
 EventLoop::~EventLoop() noexcept
 {
-	assert(timers.empty());
 	assert(defer.empty());
 	assert(idle.empty());
 #ifdef HAVE_THREADED_EVENT_LOOP
@@ -152,12 +143,18 @@ EventLoop::AbandonFD(SocketEvent &event)  noexcept
 }
 
 void
-EventLoop::AddTimer(TimerEvent &t, Event::Duration d) noexcept
+EventLoop::Insert(CoarseTimerEvent &t) noexcept
+{
+	coarse_timers.Insert(t);
+	again = true;
+}
+
+void
+EventLoop::Insert(FineTimerEvent &t) noexcept
 {
 	assert(IsInside());
 
-	t.due = SteadyNow() + d;
-	timers.insert(t);
+	timers.Insert(t);
 	again = true;
 }
 
@@ -166,24 +163,13 @@ EventLoop::HandleTimers() noexcept
 {
 	const auto now = SteadyNow();
 
-	Event::Duration timeout;
+	auto fine_timeout = timers.Run(now);
+	auto coarse_timeout = coarse_timers.Run(now);
 
-	while (!quit) {
-		auto i = timers.begin();
-		if (i == timers.end())
-			break;
-
-		TimerEvent &t = *i;
-		timeout = t.due - now;
-		if (timeout > timeout.zero())
-			return timeout;
-
-		timers.erase(i);
-
-		t.Run();
-	}
-
-	return Event::Duration(-1);
+	return fine_timeout.count() < 0 ||
+		(coarse_timeout.count() >= 0 && coarse_timeout < fine_timeout)
+		? coarse_timeout
+		: fine_timeout;
 }
 
 void
