@@ -20,7 +20,7 @@
 #include "io/Path.hxx"
 #include "io/UniqueFileDescriptor.hxx"
 #include "event/SocketEvent.hxx"
-#include "event/TimerEvent.hxx"
+#include "event/CoarseTimerEvent.hxx"
 #include "util/ScopeExit.hxx"
 #include "util/UriUtil.hxx"
 
@@ -111,7 +111,7 @@ struct PluginCycle {
 	/** list of all error messages from failed plugins */
 	std::string all_errors;
 
-	TimerEvent delayed_fail_timer;
+	CoarseTimerEvent delayed_fail_timer;
 
 	PluginCycle(EventLoop &event_loop,
 		    const PluginList &_list, std::unique_ptr<char *[]> &&_argv,
@@ -341,12 +341,12 @@ make_argv(const char*const* args) noexcept
 
 PluginCycle *
 plugin_run(EventLoop &event_loop,
-	   PluginList *list, const char *const*args,
+	   const PluginList &list, const char *const*args,
 	   PluginResponseHandler &handler) noexcept
 {
 	assert(args != nullptr);
 
-	auto *cycle = new PluginCycle(event_loop, *list, make_argv(args),
+	auto *cycle = new PluginCycle(event_loop, list, make_argv(args),
 				      handler);
 	cycle->TryNextPlugin();
 
@@ -372,7 +372,7 @@ PluginCycle::Stop() noexcept
 }
 
 void
-plugin_stop(PluginCycle *cycle) noexcept
+plugin_stop(PluginCycle &cycle) noexcept
 {
-	cycle->Stop();
+	cycle.Stop();
 }
