@@ -1,6 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2020 The Music Player Daemon Project
- * Project homepage: http://musicpd.org
+ * Copyright 2004-2021 The Music Player Daemon Project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,8 +27,8 @@
 static const unsigned SCREEN_MIN_COLS = 14;
 static const unsigned SCREEN_MIN_ROWS = 5;
 
-ScreenManager::ScreenManager(EventLoop &_event_loop) noexcept
-	:event_loop(_event_loop),
+ScreenManager::ScreenManager(EventLoop &event_loop) noexcept
+	:paint_event(event_loop, BIND_THIS_METHOD(Paint)),
 	 layout({std::max<unsigned>(COLS, SCREEN_MIN_COLS),
 		 std::max<unsigned>(LINES, SCREEN_MIN_ROWS)}),
 	 title_bar({layout.title_x, layout.title_y}, layout.size.width),
@@ -105,7 +104,7 @@ ScreenManager::OnResize() noexcept
 	curs_set(1);
 	curs_set(0);
 
-	Paint(true);
+	SchedulePaint();
 }
 
 void
