@@ -1,6 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2020 The Music Player Daemon Project
- * Project homepage: http://musicpd.org
+ * Copyright 2004-2021 The Music Player Daemon Project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +25,7 @@
 #include "SearchPage.hxx"
 #include "SongPage.hxx"
 #include "KeyDefPage.hxx"
+#include "EditPlaylistPage.hxx"
 #include "LyricsPage.hxx"
 #include "OutputsPage.hxx"
 #include "ChatPage.hxx"
@@ -62,6 +62,9 @@ static const PageMeta *const screens[] = {
 #ifdef ENABLE_KEYDEF_SCREEN
 	&screen_keydef,
 #endif
+#ifdef ENABLE_PLAYLIST_EDITOR
+	&edit_playlist_page,
+#endif
 };
 
 const PageMeta *
