@@ -1,20 +1,20 @@
 #include "BasicMarquee.hxx"
 #include "config.h"
 
-#include <glib.h>
 #include <stdlib.h>
 
 #ifdef ENABLE_LOCALE
 #include <locale.h>
 #endif
 
+#include <stdio.h>
+
 int main(int argc, char **argv)
 {
-	char *p;
 	unsigned width, count;
 
 	if (argc != 5) {
-		g_printerr("Usage: %s TEXT SEPARATOR WIDTH COUNT\n", argv[0]);
+		fprintf(stderr, "Usage: %s TEXT SEPARATOR WIDTH COUNT\n", argv[0]);
 		return 1;
 	}
 
@@ -29,9 +29,8 @@ int main(int argc, char **argv)
 	hscroll.Set(width, argv[1]);
 
 	for (unsigned i = 0; i < count; ++i) {
-		p = hscroll.ScrollString();
-		g_print("%s\n", p);
-		g_free(p);
+		const auto s = hscroll.ScrollString();
+		fprintf(stderr, "%.*s\n", int(s.second), s.first);
 
 		hscroll.Step();
 	}
